//******************************************
//  MSP-FET430F1121- Numrotation dcimale *
//******************************************
// Cblage :
//  - prise de ligne PLIGNE sur P2.3
//  - P2.2 reli  P2.3 pour provoquer une int au flanc montant
//  - bouton poussoir sur P2.1 (niveau actif = "0")

#include  <msp430x11x1.h>

// Variables
char CHIFFRE; // Chiffre compos
const char *NUMERO; // Adresse du numro compos
int IND_NUM;  // Index dans NUMERO
// Constantes
const char NUMERO1[] = {2,3,4,5,0xFF};// 0xFF : indicateur de fin
const char NUMERO2[] = {10,3,8,8,4,10,5,2,5,2,0xFF};// 0xFF : indicateur de fin
const char NUMERO3[] = {2,3,0xFF};// 0xFF : indicateur de fin
#define K1000mS 32767;
#define K966mS  31686;
#define K100mS  3277;
#define K66mS   2196;

// Programme d'interruption sur P2 (ici P2.2 reli  P2.3 = PLIGNE)
// Activ au flanc montant, donc  la fermeture de ligne
// Fonctions : - affecter convenablement CCR0 et CCR1 :
//               - pour ouvrir la ligne pdt 67mS pdt la composition du chiffre
//                 et la fermer 33mS plus tard
//               - pour ouvrir la ligne pdt 967mS entre 2 chiffres
//                 et la fermer 33mS plus tard
//             - lire le prochain chiffre du numro
//             - bloquer compteur TAR et inhiber int. si dernier chiffre compos
#pragma vector=PORT2_VECTOR
__interrupt void INT_P2()
{
  CHIFFRE = CHIFFRE - 1; // Comptage des impulsions
  if (CHIFFRE != 0)      // Dernire impulsion du chiffre ?
   {               // Non
    CCR0 = K100mS; // Priode de comptage = 100ms (pour une horloge ACLK de 32768Hz)
    CCR1 = K66mS;  // Reset OUT1=P2.3  66mS (pour une horloge ACLK de 32768Hz)
   }
  else
   {                // Oui
    CCR1 = K966mS;  // Reset OUT1=P2.3  966mS (pour une horloge ACLK de 32768Hz)
    CCR0 = K1000mS; // Set OUT1=P2.3  1000mS (fermeture de ligne)
    IND_NUM = IND_NUM + 1;     // Pour pointer le chiffre suivant
    CHIFFRE = NUMERO[IND_NUM]; // Chiffre suivant
    if (CHIFFRE == 0xFF)       // Dernier chiffre du numro ?
     {
      TACTL = 0x0104; // Blocage et Raz TAR
      P2IE &= 0xFB;   // Inhibition interruptions sur P2.2
     }
   }
  P2IFG &= 0xFB; // Acquitement interruption P2.2
}

void Compose_numero(void)
 {
  IND_NUM = 0;    // On commence par le 1 chiffre du numro
  NUMERO = NUMERO1;
  CHIFFRE = NUMERO[IND_NUM]; // 1 chiffre du numro
  CCR1 = K966mS;  // Reset OUT1=P2.3  966mS (pour une horloge ACLK de 32768Hz)
  CCR0 = K1000mS; // Set OUT1=P2.3  1000mS (fermeture de ligne)
  TACTL = 0x0114; // Comptage "up" modulo CCR0 et Raz TAR
  P2IFG &= 0xFB; // Raz indicateur interruption P2.2
  P2IE |= 0x04;  // Validation interruptions sur P2.2
 }

void Prise_ligne(void)
 {
  CCR1 = 0xFFFF;  // CCR1 jamais atteint car > CCR0
  CCR0 = 1;       // Set OUT1=P2.3  1/32768=30S (fermeture de ligne)
  TACTL = 0x0114; // Comptage "up" modulo CCR0 et Raz TAR
 }

void main(void)
{
  WDTCTL = WDTPW + WDTHOLD; // Arrt du chien de garde
  P2DIR = 0x08;   // P2.3 en sortie et P2.2 en entre
  P2SEL = 0x08;   // OUT1 du TimerA aiguill sur P2.3
  CCTL1 = 0x00E0; // Configuration du timer : Capture inhibe
                  //                          Mode "Compare"
                  //                          Output mode = 7 (EQU1 reset OUT1, EQU0 set OUT1)
  // Initialisation TimerA pour respecter l'intervalle inter-chiffre  la 1 int
  TACTL = 0x0104; // Configuration du timer : TAR bloqu
                  //                          Horloge = ACLK =32768Hz
                  //                          Prdiviseur = 1
                  //                          RAZ du compteur TAR
                  //                          Interruptions inhibes
  // Autres initialisations
  P2IES = 0;      // Interruption sur flanc montant de P2.2
  P2IFG = 0;      // Raz de tous les indicateurs INT de P2
  _EINT();        // Validation interruptions ds CPU

  // Boucle sans fin
  for (;;)
  {
    if ((P2IN & 2) == 0) // Action sur bouton poussoir ?
    {
     Prise_ligne();
     // Test numrotation
     Compose_numero();
    }
  }
}
