#include  <msp430x11x1.h>

// Constantes
#define K1000mS 32767;
#define K967mS  31686;
#define K100mS  3277;
#define K67mS   2196;

// Variables
char CHIFFRE;    // Dclaration d'une variable en RAM de taille 1 octet

// Programme principal
// *******************
void main(void)
 {
  WDTCTL = WDTPW + WDTHOLD; // Arrt du chien de garde
  P2DIR = 1 | 8 | 0x10 ; // P2.0, P2.3 et P2.4 en sortie; P2.1 & P2.2 en entre
  P2SEL = 8 ; // OUT1 du TimerA aiguill sur P2.3
  P2IE |= 2 ; // Validation interruptions sur P2.1
  P2IES|= 2 ; // Slection flanc descendant sur P2.1
  CCTL1 = 0x00E0; // Configuration du timer : Capture inhibe
                  //                          Mode "Compare"
                  //                          Output mode = 7 (EQU1 reset OUT1, EQU0 set OUT1)
  _EINT();        // Validation interruptions ds CPU
  for (;;)
  {
   P2OUT ^= 1 ; ;// Inversion du bit P2.0
  }
 }

 // Programmes d'interruption sur P2
// ********************************
// P2.2 reli  P2.3 = PLIGNE : activ au flanc montant, donc  la fermeture de ligne
// P2.1 = BP activ au flanc descendant, donc  l'appui du bouton poussoir
#pragma vector=PORT2_VECTOR
__interrupt void INT_P2()
 {
  switch (P2IFG & 0x06)
  {
   case 0x02 : // Flanc descendant sur P2.1 ? (fonction FS1)
    {
     // Test numrotation
     CHIFFRE = 7;    // Chiffre  composer
     CCR1  = K967mS ;// Reset OUT1=P2.3  967mS (pour une horloge ACLK de 32768Hz)
     CCR0  = K1000mS;// Set OUT1=P2.3  1000mS (fermeture de ligne)
     TACTL = 0x0114; // Configuration du timer : Comptage "up" modulo CCR0
                     //                          Horloge = ACLK =32768Hz
                     //                          Prdiviseur = 1
                     //                          RAZ du compteur TAR
                     //                          Interruptions inhibes
     P2IFG &= 255-4  ;// Raz indicateur interruption P2.2
     P2IE  |= 4      ;// Validation interruptions sur P2.2
     P2IFG &= 255-2 ;// Acquitement interruption P2.1
     break;
    }
   case 0x04 : // Flanc montant sur P2.2 ?
    {
     CCR1 = K67mS ; // Reset OUT1=P2.3  67mS (pour une horloge ACLK de 32768Hz)
     CCR0 = K100mS; // Set OUT1=P2.3  100mS (pour une horloge ACLK de 32768Hz)
     CHIFFRE = CHIFFRE - 1; // Comptage des impulsions du chiffre
     if (CHIFFRE == 0)      // Dernire impulsion du chiffre ?
      {
       TACTL = 0x0104 ; // Blocage et Raz TAR
       P2IE &= 255-4  ; // Inhibition interruptions sur P2.2
      }
     P2IFG &= 255-4;    // Acquitement interruption P2.2
     break;
    }
  }
 }
