#include  <msp430x11x1.h>

// Constantes
#define K1000mS 32767;
#define K967mS  31686;
#define K100mS  3277;
#define K67mS   2196;

// Constantes en mmoire morte
const char NUMERO[] = {10,3,8,8,4,10,5,2,5,2,0xFF};// 0xFF : indicateur de fin

// Variables en RAM
char CHIFFRE;
const char *CHIFFRE_ACT; // Pointeur du chiffre ds le numro actuellement compos

void main(void)
 {
  WDTCTL = WDTPW + WDTHOLD; // Arrt du chien de garde
  P2DIR = 1 | 8 | 0x10; // P2.0, P2.3 et P2.4 en sortie; P2.1 & P2.2 en entre
  P2SEL = 8; // OUT1 du TimerA aiguill sur P2.3
  P2IE |= 2; // Validation interruptions sur P2.1
  P2IES|= 2; // Slection flanc descendant sur P2.1
  CCTL1 = 0x00E0; // Configuration du timer : ____________________________
  _EINT();        // Validation interruptions ds CPU
  for (;;)
  {
   P2OUT ^= 1; ;// Inversion du bit P2.0
  }
 }
 // Fonctions
void Prise_ligne(void)
 {
  CCR1 = 0xFFFF;  // CCR1 jamais atteint car > CCR0
  CCR0 = 1;       // Set OUT1=P2.3  1/32768=30S (fermeture de ligne)
  TACTL = 0x0114; // Comptage "up" modulo CCR0 et Raz TAR
 }

void Compose_numero(const char *NUMERO)
 {
  CHIFFRE_ACT = NUMERO;   // Pointe le 1 chiffre du numro
  CHIFFRE = *CHIFFRE_ACT; // 1 chiffre du numro
  CCR1 = K967mS;  // Reset OUT1=P2.3  966mS (pour une horloge ACLK de 32768Hz)
  CCR0 = K1000mS; // Set OUT1=P2.3  1000mS (fermeture de ligne)
  TACTL = 0x0114; // Comptage "up" modulo CCR0 et Raz TAR
 }

// Programmes d'interruption sur P2
// P2.2 reli  P2.3 = PLIGNE : activ au flanc montant, donc  la fermeture de ligne
// P2.1 = BP activ au flanc descendant
#pragma vector=PORT2_VECTOR
__interrupt void INT_P2()
 {
  switch (P2IFG & 0x06)
  {
   case 0x02 : // Flanc descendant sur P2.1
    // Test numrotation
    Prise_ligne();
    Compose_numero(&NUMERO[0]);
    P2IE  |= 4; // Validation interruptions sur P2.2
    P2IFG &= 255-2-4; // Acquitement interruption P2.1 et P2.2
    break;
   case 0x04 : // Flanc montant sur P2.2
    CHIFFRE = CHIFFRE - 1; // Comptage des impulsions
    if (CHIFFRE != 0)      // Dernire impulsion du chiffre ?
     {               // Non
      CCR1 = K67mS ; // Reset OUT1=P2.3  67mS (pour une horloge ACLK de 32768Hz)
      CCR0 = K100mS; // Set OUT1=P2.3  100mS (pour une horloge ACLK de 32768Hz)
     }
    else
     {                // Oui
      CCR1 = K967mS;  // Reset OUT1=P2.3  967mS (pour ACLK=32768Hz)
      CCR0 = K1000mS; // Set OUT1=P2.3  1000mS (fermeture de ligne)
      CHIFFRE_ACT++;          // Pour pointer le chiffre suivant
      CHIFFRE = *CHIFFRE_ACT; // Chiffre suivant
      if (CHIFFRE == 0xFF)    // Dernier chiffre du numro ?
       {
        TACTL = 0x0104; // Blocage et Raz TAR
        P2IE |= 2; // Validation interruptions sur P2.1
       }
     }
    P2IFG &= 255-4 ;    // Acquitement interruption P2.2
    break;
  }
 }
